@@ -28,6 +28,8 @@ use warnings;
 
 use File::Spec;
 use File::Path qw(mkpath);
+use File::Copy qw(mv);
+use File::Basename qw(dirname basename);
 
 use TigerVNC::OptionTie;
 use TigerVNC::Common;
@@ -200,26 +202,7 @@ sub handleVNCStartupAuto {
   my ($vncStartup, $options) = @_;
 
   my $vncStartupAuto = $vncStartup;
-  if (-f File::Spec->catfile($options->{'vncUserDir'}, "Xtigervnc-session")) {
-    # A user provided Xtigervnc-session script exists => use it.
-    $vncStartupAuto =
-      File::Spec->catfile($options->{'vncUserDir'}, "Xtigervnc-session");
-  } elsif (-f File::Spec->catfile($options->{'vncUserDir'}, "Xvnc-session")) {
-    # This is deprecated rename it to Xtigervnc-session.
-    if (rename
-        File::Spec->catfile($options->{'vncUserDir'}, "Xvnc-session")
-      , File::Spec->catfile($options->{'vncUserDir'}, "Xtigervnc-session")) {
-      symlink "Xtigervnc-session"
-        , File::Spec->catfile($options->{'vncUserDir'}, "Xvnc-session");
-      # A user provided Xtigervnc-session script exists => use it.
-      $vncStartupAuto =
-        File::Spec->catfile($options->{'vncUserDir'}, "Xtigervnc-session");
-    } else {
-      # A user provided Xtigervnc-session script exists => use it.
-      $vncStartupAuto =
-        File::Spec->catfile($options->{'vncUserDir'}, "Xvnc-session");
-    }
-  } elsif (-f File::Spec->catfile($options->{'vncUserDir'}, "xstartup")) {
+  if (-f File::Spec->catfile($options->{'vncUserDir'}, "xstartup")) {
     # A user provided Xtigervnc-session script exists => use it.
     $vncStartupAuto =
       File::Spec->catfile($options->{'vncUserDir'}, "xstartup");
@@ -369,6 +352,8 @@ sub getOptionParseTable($$) {
        "if specified, debugging output is enabled." ],
       [36, 'dry-run'           => 'dry-run',
        "if enabled, no real action is taken only a simulation of what would be done is performed." ],
+      [36, 'pidfile=s'         => 'vncPidFile',
+       "file to store the pid of the running $server server." ],
       [36, 'I-KNOW-THIS-IS-INSECURE' => 'I-KNOW-THIS-IS-INSECURE' ],
       # Parameters for both Xtigervnc and X0tigervnc (case insensitive)
       [63, 'ZlibLevel=i'       => 'ZlibLevel' ],
@@ -501,6 +486,7 @@ sub getOptionParseTable($$) {
       # Backward compatible configuration file option for fp
       [ 2, 'fontPath=s'        => 'fontPath' ],
       # Options only for tigervncserver
+      [14, 'query=s'           => 'query' ],
       [14, 'geometry=s'        => sub {
           if (@_ == 2) {
             my $geometry = $_[1] // "undef";
@@ -572,6 +558,7 @@ sub getOptionParseTable($$) {
       [ 4, 'noxstartup'        => sub {
           if (@_ == 2) {
             &{$override}('vncStartup', undef);
+            &{$override}('autokill', 0);
           } elsif (defined $options->{'src'}->{'vncStartup'}) {
             return !defined $options->{'vncStartup'};
           } else {
@@ -779,22 +766,9 @@ sub readConfigFile($$) {
     $configFile = File::Spec->catfile($SYSTEMCONFIGDIR,
       "vncserver-config-defaults");
   } elsif ($desc eq 'user') {
-    if (-f File::Spec->catfile($options->{'vncUserDir'}, "tigervnc.conf")) {
+    if (-f File::Spec->catfile($options->{'vncUserDir'}, "config.pl")) {
       # User provided TigerVNC configuration found
-      $configFile = File::Spec->catfile($options->{'vncUserDir'}, "tigervnc.conf");
-    } elsif (-f File::Spec->catfile($options->{'vncUserDir'}, "vnc.conf")) {
-      # This is deprecated rename it to tigervnc.conf
-      if (rename
-          File::Spec->catfile($options->{'vncUserDir'}, "vnc.conf")
-        , File::Spec->catfile($options->{'vncUserDir'}, "tigervnc.conf")) {
-        symlink "tigervnc.conf"
-          , File::Spec->catfile($options->{'vncUserDir'}, "vnc.conf");
-        # User provided TigerVNC configuration found
-        $configFile = File::Spec->catfile($options->{'vncUserDir'}, "tigervnc.conf");
-      } else {
-        # User provided TigerVNC configuration found
-        $configFile = File::Spec->catfile($options->{'vncUserDir'}, "vnc.conf");
-      }
+      $configFile = File::Spec->catfile($options->{'vncUserDir'}, "config.pl");
     } elsif (-f File::Spec->catfile($options->{'vncUserDir'}, "config")) {
       # User provided TigerVNC configuration found
       $configFile = File::Spec->catfile($options->{'vncUserDir'}, "config");
@@ -1393,6 +1367,10 @@ sub getConfig {
 ## Values declared as user values in /etc/tigervnc/vncserver-config-*, i.e.,
 ## values that are intended to be overwritten by ~/.vnc/tigervnc.conf.
       vncUserDir                =>
+        defined($ENV{XDG_CONFIG_HOME}) && rindex($ENV{XDG_CONFIG_HOME}, "/", 0) == 0
+        ? File::Spec->catfile($ENV{XDG_CONFIG_HOME}, "tigervnc")
+        : File::Spec->catfile($ENV{HOME}, ".config", "tigervnc"),
+      vncLegacyDir              =>
         File::Spec->catfile($ENV{HOME}, ".vnc"),
       vncPasswdFile             =>
         undef, # later derived from vncUserDir
@@ -1423,7 +1401,7 @@ sub getConfig {
       RSAKey                    =>
         undef, # auto generated if absent and stored in
                # ~/.vnc/${HOSTFQDN}-SrvRsaKey.pem
-      RequireUsername		=> undef,
+      RequireUsername           => undef,
       session                   => undef,
 # Undocumented values
       vncServerExtraArgs        => [],
@@ -1450,7 +1428,54 @@ sub getConfig {
       print STDERR "$PROG: Could not create $opts->{'vncUserDir'}, file exists but is not a directory.\n";
       exit 1;
     }
-    if (!mkpath ($opts->{'vncUserDir'}, 0, 0755)) {
+    # legacy ~/.vnc directory migration code
+    if (!(-l $opts->{'vncLegacyDir'}) && (-d $opts->{'vncLegacyDir'})) {
+      if (!mv($opts->{'vncLegacyDir'}, $opts->{'vncUserDir'})) {
+        print STDERR "$PROG: Could not migrate $opts->{'vncLegacyDir'} to $opts->{'vncUserDir'}.\n";
+        exit 1;
+      }
+      symlink File::Spec->abs2rel(
+            $opts->{'vncUserDir'}, dirname($opts->{'vncLegacyDir'}))
+        , $opts->{'vncLegacyDir'};
+      if (!-f File::Spec->catfile($opts->{'vncUserDir'}, "config.pl")) {
+        if (-f File::Spec->catfile($opts->{'vncUserDir'}, "tigervnc.conf")) {
+          # This is deprecated rename it to config.pl
+          if (rename
+              File::Spec->catfile($opts->{'vncUserDir'}, "tigervnc.conf")
+            , File::Spec->catfile($opts->{'vncUserDir'}, "config.pl")) {
+            symlink "config.pl"
+              , File::Spec->catfile($opts->{'vncUserDir'}, "tigervnc.conf");
+          }
+        } elsif (-f File::Spec->catfile($opts->{'vncUserDir'}, "vnc.conf")) {
+          # This is deprecated rename it to config.pl
+          if (rename
+              File::Spec->catfile($opts->{'vncUserDir'}, "vnc.conf")
+            , File::Spec->catfile($opts->{'vncUserDir'}, "config.pl")) {
+            symlink "config.pl"
+              , File::Spec->catfile($opts->{'vncUserDir'}, "vnc.conf");
+          }
+        }
+      }
+      if (!-f File::Spec->catfile($opts->{'vncUserDir'}, "xstartup")) {
+        if (-f File::Spec->catfile($opts->{'vncUserDir'}, "Xtigervnc-session")) {
+          # This is deprecated rename it to xstartup
+          if (rename
+              File::Spec->catfile($opts->{'vncUserDir'}, "Xtigervnc-session")
+            , File::Spec->catfile($opts->{'vncUserDir'}, "xstartup")) {
+            symlink "xstartup"
+              , File::Spec->catfile($opts->{'vncUserDir'}, "Xtigervnc-session");
+          }
+        } elsif (-f File::Spec->catfile($opts->{'vncUserDir'}, "Xvnc-session")) {
+          # This is deprecated rename it to xstartup
+          if (rename
+              File::Spec->catfile($opts->{'vncUserDir'}, "Xvnc-session")
+            , File::Spec->catfile($opts->{'vncUserDir'}, "xstartup")) {
+            symlink "xstartup"
+              , File::Spec->catfile($opts->{'vncUserDir'}, "Xvnc-session");
+          }
+        }
+      }
+    } elsif (!mkpath ($opts->{'vncUserDir'}, 0, 0755)) {
       print STDERR "$PROG: Could not create $opts->{'vncUserDir'}.\n";
       exit 1;
     }
